@@ -102,8 +102,15 @@ init ()
   if (1 == once)
     return;
   once = 1;
+  const struct GNUNET_OS_ProjectData *pd = GNUNET_OS_project_data_get ();
+  const struct GNUNET_OS_ProjectData *dpd = GNUNET_OS_project_data_default ();
+
+  if (pd != dpd)
+    GNUNET_OS_init (dpd);
   GNUNET_PLUGIN_load_all ("libgnunet_plugin_gnsrecord_", NULL,
                           &add_plugin, NULL);
+  if (pd != dpd)
+    GNUNET_OS_init (pd);
 }
 
 
@@ -114,6 +121,11 @@ void __attribute__ ((destructor))
 GNSRECORD_fini ()
 {
   struct Plugin *plugin;
+  const struct GNUNET_OS_ProjectData *pd = GNUNET_OS_project_data_get ();
+  const struct GNUNET_OS_ProjectData *dpd = GNUNET_OS_project_data_default ();
+
+  if (pd != dpd)
+    GNUNET_OS_init (dpd);
 
   for (unsigned int i = 0; i < num_plugins; i++)
   {
@@ -125,6 +137,10 @@ GNSRECORD_fini ()
     GNUNET_free (plugin);
   }
   GNUNET_free (gns_plugins);
+
+  if (pd != dpd)
+    GNUNET_OS_init (pd);
+
   gns_plugins = NULL;
   once = 0;
   num_plugins = 0;
