@@ -915,7 +915,7 @@ refresh_block (struct NamestoreClient *nc,
   nick = get_nick_record (zone_key);
   res_count = rd_count;
   res = (struct GNUNET_GNSRECORD_Data *) rd;  /* fixme: a bit unclean... */
-  if (NULL != nick  && (0 != strcmp (name, GNUNET_GNS_EMPTY_LABEL_AT)))
+  if ((NULL != nick) && (0 != strcmp (name, GNUNET_GNS_EMPTY_LABEL_AT)))
   {
     nick->flags =
       (nick->flags | GNUNET_GNSRECORD_RF_PRIVATE) ^ GNUNET_GNSRECORD_RF_PRIVATE;
@@ -1809,7 +1809,7 @@ run_zone_iteration_round (struct ZoneIteration *zi, uint64_t limit)
   start = GNUNET_TIME_absolute_get ();
   GNUNET_break (GNUNET_SYSERR !=
                 GSN_database->iterate_records (GSN_database->cls,
-                                               (0 == GNUNET_is_zero (&zi->zone))
+                                               (GNUNET_YES == GNUNET_is_zero (&zi->zone))
                                                ? NULL
                                                : &zi->zone,
                                                zi->seq,
@@ -2083,7 +2083,7 @@ monitor_iteration_next (void *cls)
   else
     zm->iteration_cnt = zm->limit; /* use it all */
   ret = GSN_database->iterate_records (GSN_database->cls,
-                                       (0 == GNUNET_is_zero (&zm->zone))
+                                       (GNUNET_YES == GNUNET_is_zero (&zm->zone))
                                        ? NULL
                                        : &zm->zone,
                                        zm->seq,
